// Sketch to trigger sound using basic cadmium-sulphide LDR. There was no data sheet accompanying this LDR, so it was tested in light
// and dark conditions, first with a multimeter, and then on the Arduino. It provided a wide range of values, which is greate for 
// selecting a threshold. In this case, a lower threshold of 300 was chosen for pausing/restarting the wave sound, and upper threshold
// of 400 chosen for initially starting play and resuming play. The max value was around 900 and minimum (zero light) around 0.
//
// There is a gap between thresholds (as you'll notice) and this was to prevent the wave file from jumping between on and off, which 
// would occur if you chose a single threshold value. i.e. if you chose, say, 300 as the cutoff between playing and pausing, then when
// the light condition is around this value, the wave file will continually jump between playing and then pausing.


#include <FatReader.h>
#include <SdReader.h>
#include <avr/pgmspace.h>
#include "WaveUtil.h"
#include "WaveHC.h"


SdReader card;    // This object holds the information for the card
FatVolume vol;    // This holds the information for the partition on the card
FatReader root;   // This holds the information for the filesystem on the card
FatReader f;      // This holds the information for the file we're play

WaveHC wave;      // This is the only wave (audio) object, since we will only play one at a time

const int SENSORPIN = 0;
const int LOWER_THRESHOLD = 300;  // Pause or restart play
const int UPPER_THRESHOLD = 400;  // Start or resume playing when this value or higher
char* FILENAME = "BARORGAN.WAV";  // Name of the wave file to play

// this handy function will return the number of bytes currently free in RAM, great for debugging!   
int freeRam(void)
{
  extern int  __bss_end; 
  extern int  *__brkval; 
  int free_memory; 
  if((int)__brkval == 0) {
    free_memory = ((int)&free_memory) - ((int)&__bss_end); 
  }
  else {
    free_memory = ((int)&free_memory) - ((int)__brkval); 
  }
  return free_memory; 
} 

void sdErrorCheck(void)
{
  if (!card.errorCode()) return;
  putstring("\n\rSD I/O error: ");
  Serial.print(card.errorCode(), HEX);
  putstring(", ");
  Serial.println(card.errorData(), HEX);
  while(1);
}

// Initilaise the SD card
void initCard() {
  //  if (!card.init(true)) { //play with 4 MHz spi if 8MHz isn't working for you
  if (!card.init()) {         //play with 8 MHz spi (default faster!)  
    putstring_nl("Card init. failed!");  // Something went wrong, lets print out why
    sdErrorCheck();
    while(1);                            // then 'halt' - do nothing!
  }
  
  // enable optimize read - some cards may timeout. Disable if you're having problems
  card.partialBlockRead(true);
 
// Now we will look for a FAT partition!
  uint8_t part;
  for (part = 0; part < 5; part++) {     // we have up to 5 slots to look in
    if (vol.init(card, part)) 
      break;                             // we found one, lets bail
  }
  if (part == 5) {                       // if we ended up not finding one  :(
    putstring_nl("No valid FAT partition!");
    sdErrorCheck();      // Something went wrong, lets print out why
    while(1);                            // then 'halt' - do nothing!
  }
  
  // Lets tell the user about what we found
  putstring("Using partition ");
  Serial.print(part, DEC);
  putstring(", type is FAT");
  Serial.println(vol.fatType(),DEC);     // FAT16 or FAT32?
  
  // Try to open the root directory
  if (!root.openRoot(vol)) {
    putstring_nl("Can't open root dir!"); // Something went wrong,
    while(1);                             // then 'halt' - do nothing!
  }  
}

void setup() {
  byte i;
  
  // set up serial port
  Serial.begin(9600);

  putstring("Free RAM: ");       // This can help with debugging, running out of RAM is bad
  Serial.println(freeRam());      // if this is under 150 bytes it may spell trouble!
  
  // Set the output pins for the DAC control. This pins are defined in the library
  pinMode(2, OUTPUT);
  pinMode(3, OUTPUT);
  pinMode(4, OUTPUT);
  pinMode(5, OUTPUT);
  
  // Initialise the SD card
  initCard();
  
}

void loop() {
  int rate = analogRead(SENSORPIN);
  
  if(rate < LOWER_THRESHOLD) {
    if (wave.isplaying) {
      wave.pause();
    } else {
      wave.seek(0); 
    }
  }
  
  if (rate > UPPER_THRESHOLD) {
    if (!wave.isplaying) {
      playfile(FILENAME);
    } else {
      wave.resume(); 
    }
  }
  
  // View light values (test line)
  //Serial.println(rate);
}

void playfile(char *name) {
  // see if the wave object is currently doing something
  if (wave.isplaying) {// already playing something, so stop it!
    wave.stop(); // stop it
  }
  // look in the root directory and open the file
  if (!f.open(root, name)) {
    putstring("Couldn't open file "); Serial.print(name); return;
  }
  // OK read the file and turn it into a wave object
  if (!wave.create(f)) {
    putstring_nl("Not a valid WAV"); return;
  }
  
  // ok time to play! start playback
  wave.play();
}
